package com.malk.service.dingtalk.impl;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.dingtalk.DDR;
import com.malk.service.dingtalk.DDClient_Personnel;
import com.malk.utils.UtilList;
import com.malk.utils.UtilMap;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class DDImplClient_Personnel implements DDClient_Personnel {

    private final int size = 50;

    private final int offset = 0;

    /**
     * 获取花名册元数据
     */
    @Override
    public List<Map> getPersonnelMeta(String access_token, Number agentid) {
        return (List<Map>) DDR.doPost("https://oapi.dingtalk.com/topapi/smartwork/hrm/roster/meta/get", null, DDConf.initTokenParams(access_token), UtilMap.map("agentid", agentid)).getResult();
    }

    /**
     * 获取员工花名册字段信息
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/intelligent-personnel-obtain-employee-roster-information
     */
    @Override
    public List<Map> getEmployeeInfos(String access_token, List<String> userIds, Number agentId, List<String> field_filter_list) {

        int start = 0;
        boolean isNext = true;
        List<Map> result = new ArrayList<>();
        StringBuilder ids = new StringBuilder();
        StringBuilder filters = new StringBuilder();
        if (ObjectUtil.isNotNull(field_filter_list)){
            filters.append(String.join(",",field_filter_list));
        }
        while (isNext){
            ids.append(String.join(",", userIds.subList(start * size, Math.min((start + 1) * size, userIds.size()))));
            Map bodys = UtilMap.map("userid_list, agentid", ids.toString(), agentId);
            if (ObjectUtil.isNotEmpty(filters)){
                bodys.put("field_filter_list",filters);
            }
            List<Map> dataList = (List<Map>) DDR.doPost("https://oapi.dingtalk.com/topapi/smartwork/hrm/employee/v2/list", null,  DDConf.initTokenParams(access_token), bodys).getResult();
            if (ObjectUtil.isNotNull(dataList) && dataList.size()>0){
                result.addAll(dataList);
            }
            if (userIds.size() <= ((start + 1)*size)){
                isNext = false;
            }else {
                start++;
            }
            ids = new StringBuilder();

        }
        return result;
    }

    /**
     * 获取待入职员工列表
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/intelligent-personnel-obtain-employee-roster-information
     */
    @Override
    public List<String> getPendingEmployeeIds(String access_token) {
        boolean isNext = true;
        List<String> dataList = new ArrayList<>();
        Map bodys = UtilMap.map("offset, size", offset, size );
        while (isNext){
            DDR ddr = DDR.doPost("https://oapi.dingtalk.com/topapi/smartwork/hrm/employee/querypreentry", null, DDConf.initTokenParams(access_token), bodys);
            if (ObjectUtil.isNotNull(ddr.getResult())){
                JSONObject jsonObject = JSONObject.parseObject(JSON.toJSONString(ddr.getResult()));
                JSONArray jsonArray = JSONArray.parseArray(JSON.toJSONString(jsonObject.get("data_list")));
                if (ObjectUtil.isNull(jsonObject.get("next_cursor"))){
                    isNext = false;
                }else {
                    int next = Integer.parseInt(jsonObject.get("next_cursor").toString());
                    bodys.put("offset",next);
                }
                jsonArray.forEach(e->{
                    dataList.add(e.toString());
                });
            }
        }
        return dataList;
    }

    /**
     * 获取在职员工列表
     *
     * @apiNote https://oapi.dingtalk.com/topapi/smartwork/hrm/employee/queryonjob
     */
    @Override
    public List<String> getWorkingEmployeeIds(String access_token, String statusList) {
        boolean isNext = true;
        List<String> dataList = new ArrayList<>();
        Map bodys = UtilMap.map("status_list, offset, size",statusList, offset, size );
        while (isNext){
            DDR ddr = DDR.doPost("https://oapi.dingtalk.com/topapi/smartwork/hrm/employee/queryonjob", null, DDConf.initTokenParams(access_token), bodys);
            JSONObject jsonObject = JSONObject.parseObject(JSON.toJSONString(ddr.getResult()));
            JSONArray jsonArray = JSONArray.parseArray(JSON.toJSONString(jsonObject.get("data_list")));
            if (ObjectUtil.isNull(jsonObject.get("next_cursor"))){
                isNext = false;
            }else {
                int next = Integer.parseInt(jsonObject.get("next_cursor").toString());
                bodys.put("offset",next);
            }
            jsonArray.forEach(e->{
                dataList.add(e.toString());
            });
        }
        return dataList;
    }

    /**
     * 获取离职员工id列表
     *
     * @apiNote https://api.dingtalk.com/v1.0/hrm/employees/dismissions?nextToken=?
     */
    @Override
    public List<String> getLeaveEmployeeIdList(String access_token) {
        log.info("正在调用钉钉后台智能人事离职人员ID接口");
        Long nextToken = 0L;
        boolean hasMore = true;
        List<String> maps = new ArrayList<>();
        Map header = DDConf.initTokenHeader(access_token);
        while (hasMore){
            DDR ddr = DDR.doGet("https://api.dingtalk.com/v1.0/hrm/employees/dismissions?nextToken=" + nextToken + "&maxResults=" + size, header, null);
            List<String> userIdList = ddr.getUserIdList();
            maps.addAll(userIdList);
            hasMore = ddr.isHasMore();
            if (hasMore){
                nextToken = ddr.getNextToken();
            }
        }
        return maps;
    }

    /**
     * 根据员工id获取离职员工信息
     *
     * @apiNote https://oapi.dingtalk.com/topapi/smartwork/hrm/employee/queryonjob
     */
    @Override
    public List<Map> getLeaveEmployeeInfos(String access_token, List<String> userIdList) {
        log.info("正在调用钉钉后台智能人事离职人员信息接口");
        List<String> idList = new ArrayList<>();
        List<Map> lastResult = new ArrayList<>();
        Map header = DDConf.initTokenHeader(access_token);
        for (int i = 0; i < userIdList.size(); i+=size) {
            idList.addAll(userIdList.subList(i,Math.min(i+size,userIdList.size())));
                List<Map> result = (List<Map>) DDR.doGet("https://api.dingtalk.com/v1.0/hrm/employees/dimissionInfos?userIdList=" + JSON.toJSONString(idList), header, null).getResult();
                if (ObjectUtil.isNotNull(result) && result.size()>0){
                    lastResult.addAll(result);
                }
            idList.clear();
        }
        return lastResult;
    }

}
